// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_multimin.h>
#include "../../config.h"
#include "fit.h"

static int unused __attribute__((unused));

void fit_read_config(struct fitvars *fitconfig)
{
char name[1000];
FILE *config;
config = fopen("./fit.inp","r");
if (config == NULL)
{
	printf("no file %s\n",name);
	exit(0);
}

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(fitconfig->simplexmul));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->simplexreset));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->fit_jv));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->fit_dark_ce_ss));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->fit_light_ce));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->fit_rbi));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->fit_time_dark));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->fit_time_light));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->tpcmax));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->dotimece));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->dotimecelight));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->tofmax));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->cemax));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->stark_max));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->stark_diff));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->starknp_max));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->shg_max));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->celiv_max));

config_read_line_to_int(&(fitconfig->pulse_max),config,"#fit_pulse");

config_read_line_to_int(&(fitconfig->pulse_voc_max),config,"#fit_pulse_voc");

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->tpv_max));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->fit_vocdarkmu));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->fit_jvtau));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(fitconfig->newton));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(fitconfig->tof_error_mull));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(fitconfig->chargedark_fitmul));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(fitconfig->chargelight_fitmul));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(fitconfig->rbi_fitmul));


unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(fitconfig->banderror_fitmul));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(fitconfig->voc_tau_mul));

unused=fscanf(config,"%s",name);

int i=0;
int items=0;
int line=0;
char file[20];
char token[40];
int pos;
int dofit;
do
{
unused=fscanf(config,"%s",token);
if (strcmp(token,"#ver")==0) break;
unused=fscanf(config,"%s",file);
pos=config_search_token(file,token);
unused=fscanf(config,"%d",&dofit);


if (dofit>0)
{
	for (i=0;i<dofit;i++)
	{
		strcpy(fitconfig->fit_file[items],file);
		fitconfig->fit_pos[items]=pos+i;
		items++;
		//printf("%d %s %s %d %d\n",items,name,file,pos+i,dofit);
	}
}else
{
	//printf("%d %s %s %d %d\n",items,name,file,pos,dofit);
}


line++;

}while(!feof(config));

double ver;
unused=fscanf(config,"%lf",&(ver));

if (ver!=1.71)
{
	printf("File compatability problem %s\n","fit.inp");
	exit(0);
}

unused=fscanf(config,"%s",name);

if (strcmp(name,"#end")!=0)
{
	printf("Error at end of config file\n");
	exit(0);
}
fitconfig->fitvars=items;
fclose(config);

}

