// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include <math.h>
#include <gsl/gsl_multimin.h>
#include "../../config.h"
#include "fit.h"

static int unused __attribute__((unused));
static char* unused_pchar __attribute__((unused));

double get_tail_error(struct fitvars *config)
{
double ret=0.0;


FILE *in;
char command[100];
char rest[100];
char file1[100];
char file2[100];
char token1[100];
char token2[100];
double min;
double max;
double adderror;
double value1;
double value2;
double range;
in=fopen("constraints.inp","r");
int added=FALSE;
printf("---------constraints-------------\n");
do
{
	unused=fscanf(in,"%s",command);
	if (strcmp(command,"#end")==0)
	{
		break;
	}else
	if (strcmp(command,"mm")==0)
	{
		added=FALSE;
		unused=fscanf(in,"%s",file1);
		unused=fscanf(in,"%s",token1);
		unused=fscanf(in,"%le",&max);
		unused=fscanf(in,"%le",&min);
		unused=fscanf(in,"%le",&adderror);
		value1=fabs(config_get_token_val(file1,token1));
		if (fabs(value1)<fabs(min))
		{
			ret+=adderror;
			added=TRUE;
			printf("Smaller than min\n");
		}

		if (fabs(value1)>fabs(max))
		{
			ret+=adderror;
			added=TRUE;
			printf("Bigger than max\n");
		}

		if (added==TRUE)
		{
		printf("mm file=%s line=%s max=%le min=%le addeeor=%le\n",file1,token1,max,min,adderror);
		}

		added=FALSE;
	}else
	if (strcmp(command,"bs")==0)
	{
		unused=fscanf(in,"%s",file1);
		unused=fscanf(in,"%s",token1);
		unused=fscanf(in,"%s",file2);
		unused=fscanf(in,"%s",token2);
		unused=fscanf(in,"%le",&adderror);

		value1=fabs(config_get_token_val(file1,token1));//fabs(read_value(file1,0,line1));
		value2=fabs(config_get_token_val(file2,token2));//fabs(read_value(file2,0,line2));
		if (value1<value2)
		{
			printf("Wrong way arround\n");
			printf("bs file1=%s token1=%s file2=%s token2=%s adderror=%le\n",file1,token1,file2,token2,adderror);
			ret+=adderror;
		}


	}else
	if (strcmp(command,"range")==0)
	{
		unused=fscanf(in,"%s",file1);
		unused=fscanf(in,"%s",token1);
		unused=fscanf(in,"%s",file2);
		unused=fscanf(in,"%s",token2);
		unused=fscanf(in,"%le",&range);
		unused=fscanf(in,"%le",&adderror);


		value1=fabs(config_get_token_val(file1,token1));//fabs(read_value(file1,0,line1));
		value2=fabs(config_get_token_val(file2,token2));//fabs(read_value(file2,0,line2));
		double min_range=value1;
		double max_range=value2;

		if (value1>value2)
		{
			min_range=value2;
			max_range=value1;
		}

		if ((min_range/max_range)<(1.0/range))
		{
			printf("range file1=%s file2=%s range=%le adderror=%le\n",file1,file2,range,adderror);
			ret+=adderror;
		}
	}else
	if (command[0]=='#')
	{
 		unused_pchar=fgets(rest,100,in);
		//printf("%s %s",command,rest);
	}else
	{
		printf("Command not recognized %s\n",command);
		exit(0);
	}

}while(!feof(in));

if (strcmp(command,"#end")!=0)
{
	printf("Error at end of constraints file\n");
	exit(0);
}

fclose(in);
printf("---------------------------------\n");

return ret;
}

