# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import pygtk
pygtk.require('2.0')
import gtk
import sys
import os
import shutil
from numpy import *
from matplotlib.figure import Figure
from numpy import arange, sin, pi
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
import gobject
import os, fnmatch
from plot import plot_gen
from util import find_data_file

#   columns
(
  COLUMN_LAYER,
  COLUMN_THICKNES,
  COLUMN_MATERIAL,
  COLUMN_START,
  COLUMN_ENABLE_GEN,
  COLUMN_EDITABLE
) = range(6)

# data
articles = []

def find_modes(path):
	pattern='light_1d_*_photons_norm.dat'
	result = []
	for root, dirs, files in os.walk(path):
		for name in files:
			if fnmatch.fnmatch(name, pattern):
				store=os.path.join(root, name)
				print "Rod=",store
				if store.endswith('_photons_norm.dat'):
   					 store = store[:-17]
				s=store.split("light_1d_")
				store = s[1]


				result.append(store)
	return result


class scan_item(gtk.CheckButton):
	name=""
	token=""
	filename=""
	line=""

class class_optical(gtk.Window):

	dump_dir="./light_dump/"
	lines=[]
	edit_list=[]

	line_number=[]
	config_file=""

	config_file="optics_epitaxy.inp"

	enabled=os.path.exists(config_file)

	file_name=""
	name=""
	visible=1

	def onclick(self, event):
		print 'button=%d, x=%d, y=%d, xdata=%f, ydata=%f'%(
		event.button, event.x, event.y, event.xdata, event.ydata)
		for i in range(0,len(self.layer_end)):
			if (self.layer_end[i]>event.xdata):
				break

		plot_gen(["./phys/"+self.layer_name[i]+"/alpha.inp"])

	def update_cb(self):
		self.cb.handler_block(self.cb_id)
		thefiles=find_modes(self.dump_dir)
		thefiles.sort()
		files_short=thefiles[::2]
		model=self.cb.get_model()
		model.clear()
		#self.cb.clear()
		self.cb.append_text("all")
		for i in range(0, len(files_short)):
			self.cb.append_text(str(files_short[i])+" nm")
		self.cb.set_active(0)
		self.cb.handler_unblock(self.cb_id)

	def __create_model(self):

		# create list store
		model = gtk.ListStore(
		    gobject.TYPE_STRING,
		    gobject.TYPE_STRING,
		    gobject.TYPE_STRING,
		    gobject.TYPE_STRING,
		    gobject.TYPE_STRING,
		    gobject.TYPE_BOOLEAN
		)

		# add items
		for item in articles:
			iter = model.append()

			model.set (iter,
			  COLUMN_LAYER, item[COLUMN_LAYER],
			  COLUMN_THICKNES, item[COLUMN_THICKNES],
			  COLUMN_MATERIAL, item[COLUMN_MATERIAL],
			  COLUMN_START, item[COLUMN_START],
			  COLUMN_ENABLE_GEN, item[COLUMN_ENABLE_GEN],
			  COLUMN_EDITABLE, item[COLUMN_EDITABLE]
			)
		return model


	def __add_columns(self, treeview):

		model = treeview.get_model()

		# Layer tag
		renderer = gtk.CellRendererText()
		renderer.connect("edited", self.on_cell_edited, model)
		renderer.set_data("column", COLUMN_LAYER)

		column = gtk.TreeViewColumn("Layer", renderer, text=COLUMN_LAYER,
				       editable=COLUMN_EDITABLE)
		treeview.append_column(column)

		# Thicknes
		renderer = gtk.CellRendererText()
		renderer.connect("edited", self.on_cell_edited, model)
		renderer.set_data("column", COLUMN_THICKNES)

		column = gtk.TreeViewColumn("Thicknes", renderer, text=COLUMN_THICKNES,
				       editable=COLUMN_EDITABLE)
		treeview.append_column(column)

		# Material file
		renderer = gtk.CellRendererText()
		renderer.connect("edited", self.on_cell_edited, model)
		renderer.set_data("column", COLUMN_MATERIAL)

		column = gtk.TreeViewColumn("Material", renderer, text=COLUMN_MATERIAL,
				       editable=COLUMN_EDITABLE)
		treeview.append_column(column)

		# Enable generation
		renderer = gtk.CellRendererText()
		renderer.connect("edited", self.on_cell_edited, model)
		renderer.set_data("column", COLUMN_ENABLE_GEN)

		column = gtk.TreeViewColumn("Enable Generation", renderer, text=COLUMN_ENABLE_GEN,
				       editable=COLUMN_EDITABLE)
		treeview.append_column(column)

		# Start
		renderer = gtk.CellRendererText()
		renderer.connect("edited", self.on_cell_edited, model)
		renderer.set_data("column", COLUMN_START)

		column = gtk.TreeViewColumn("Start", renderer, text=COLUMN_START,
				       editable=COLUMN_EDITABLE)
		treeview.append_column(column)

	def callback_save_image(self, widget):
		dialog = gtk.FileChooserDialog("Save plot",
                               None,
                               gtk.FILE_CHOOSER_ACTION_SAVE,
                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                gtk.STOCK_SAVE, gtk.RESPONSE_OK))
		dialog.set_default_response(gtk.RESPONSE_OK)
		dialog.set_action(gtk.FILE_CHOOSER_ACTION_CREATE_FOLDER)

		filter = gtk.FileFilter()
		filter.set_name("png")
		filter.add_pattern("*.png")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.fig.savefig(dialog.get_filename())

		elif response == gtk.RESPONSE_CANCEL:
		    print 'Closed'
		dialog.destroy()


	def on_add_item_clicked(self, button, treeview):
		new_item = ["#mat", "100e-9", "pcbm","0","1",True]

		selection = treeview.get_selection()
		model, iter = selection.get_selected()

		path = model.get_path(iter)[0]
		print path
		#model.remove(iter)

		#del articles[ path ]

		articles.insert(path, new_item) #append(new_item)

		iter = model.insert(path) #append()
		model.set (iter,
		    COLUMN_LAYER, new_item[COLUMN_LAYER],
		    COLUMN_THICKNES, new_item[COLUMN_THICKNES],
		    COLUMN_MATERIAL, new_item[COLUMN_MATERIAL],
		    COLUMN_START, new_item[COLUMN_START],
		    COLUMN_ENABLE_GEN, new_item[COLUMN_ENABLE_GEN],
		    COLUMN_EDITABLE, new_item[COLUMN_EDITABLE]
		)
		self.save_model(model)
		self.update_graph(model)

	def save_model(self, model):
		print "Saved"
		a = open("optics_epitaxy.inp", "w")
		a.write("#layers\n")
		a.write(str(len(model))+"\n")


		for item in model:
			a.write(item[COLUMN_LAYER]+"\n")
			a.write(item[COLUMN_THICKNES]+"\n")
			a.write(item[COLUMN_MATERIAL]+"\n")
			a.write(item[COLUMN_START]+"\n")
			a.write(item[COLUMN_ENABLE_GEN]+"\n")

		a.write("#ver\n")
		a.write("1.1\n")
		a.write("#end\n")
		a.close()

	def callback_close(self, widget, data=None):
		self.hide()
		return True

	def on_remove_item_clicked(self, button, treeview):

		selection = treeview.get_selection()
		model, iter = selection.get_selected()

		if iter:
			path = model.get_path(iter)[0]
			model.remove(iter)

			del articles[ path ]

			self.save_model(model)
			self.update_graph(model)


	def callback_refresh(self, button,treeview):

		selection = treeview.get_selection()
		model, iter = selection.get_selected()

		self.save_model(model)
		self.update_graph(model)
		self.update_cb()


	def gen_main_menu(self, window,vbox):
		accel_group = gtk.AccelGroup()


		item_factory = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)

		menu_items = (
		    ( "/_File",         None,         None, 0, "<Branch>" ),
		    ( "/File/_Save...",     None, self.callback_save_image, 0, "<StockItem>", "gtk-save" ),
		    ( "/File/Refresh",     None, self.callback_refresh, 0 , "<ImageItem>"),
		    ( "/File/Close",     "<control>Q", self.callback_close, 0, "<StockItem>", "gtk-quit" ),

		    )

		item_factory.create_items(menu_items)


		window.add_accel_group(accel_group)

		menubar=item_factory.get_widget("<main>")
		menubar.show_all()
		vbox.pack_start(menubar, False, True, 0)



	def update_graph(self,model):
		cmd = self.exe_command+' --optics'
		print cmd
		ret= os.system(cmd)
		self.fig.clf()
		self.draw_graph(model)
		self.fig.canvas.draw()

	def on_cell_edited(self, cell, path_string, new_text, model):

		iter = model.get_iter_from_string(path_string)
		path = model.get_path(iter)[0]
		column = cell.get_data("column")

		if column == COLUMN_LAYER:
			articles[path][COLUMN_LAYER] = new_text

			model.set(iter, column, articles[path][COLUMN_LAYER])

		if column == COLUMN_THICKNES:
			#old_text = model.get_value(iter, column)
			articles[path][COLUMN_THICKNES] = new_text
			print new_text
			model.set(iter, column, articles[path][COLUMN_THICKNES])

		if column == COLUMN_MATERIAL:
			print new_text
			#old_text = model.get_value(iter, column)
			articles[path][COLUMN_MATERIAL] = new_text

			model.set(iter, column, articles[path][COLUMN_MATERIAL])

		if column == COLUMN_START:
			#old_text = model.get_value(iter, column)
			articles[path][COLUMN_START] = new_text

			model.set(iter, column, articles[path][COLUMN_START])

		if column == COLUMN_ENABLE_GEN:
			#old_text = model.get_value(iter, column)
			articles[path][COLUMN_ENABLE_GEN] = new_text

			model.set(iter, column, articles[path][COLUMN_ENABLE_GEN])

		self.save_model(model)
		self.update_graph(model)
#####################################
	def callback_edit(self, widget, data=None):
		line=self.line_number[data]
		self.lines[line]=self.edit_list[data].get_text()
		self.edit_list[data].set_text(self.lines[line])
		print "Written data to", self.config_file
		a = open(self.config_file, "w")
		for i in range(0,len(self.lines)):
			a.write(self.lines[i]+"\n")
		a.close()


	def draw_graph(self,model):

		print "Drawing graph"

		self.layer_end=[]
		self.layer_name=[]

		n=0
		self.fig.clf()
		ax1 = self.fig.add_subplot(111)
		ax2 = ax1.twinx()
		x_pos=0.0
		layer=0
		color =['r','g','b','y','o','r','g','b','y','o']

		for item in model:
			if item[COLUMN_START]=="0":
				x_pos=x_pos-float(item[COLUMN_THICKNES])

		x_pos=x_pos*1e9
		for item in model:

			label=item[COLUMN_LAYER]
			layer_ticknes=item[COLUMN_THICKNES]
			layer_material=item[COLUMN_MATERIAL]
			start=item[COLUMN_START]
			enable_gen=item[COLUMN_ENABLE_GEN]

			delta=float(layer_ticknes)*1e9

			mat_file='./phys/'+layer_material+'/mat.inp'
			print mat_file
			myfile = open(mat_file)
			self.mat_file_lines = myfile.readlines()
			myfile.close()

			for ii in range(0, len(self.mat_file_lines)):
				self.mat_file_lines[ii]=self.mat_file_lines[ii].rstrip()

			x = [x_pos,x_pos+delta,x_pos+delta,x_pos] #arange(0.0,3.0,0.01)
			lumo=-float(self.mat_file_lines[1])
			lumo_delta=lumo-0.1
			Eg=float(self.mat_file_lines[3])
			homo=lumo-Eg
			homo_delta=homo-0.1
			if Eg==0.0:
				lumo_delta=lumo-0.5
				homo=0.0
			lumo_shape = [lumo,lumo,lumo_delta,lumo_delta]
			x_pos=x_pos+delta
			self.layer_end.append(x_pos)
			self.layer_name.append(layer_material)
			ax2.fill(x,lumo_shape, color[layer],alpha=0.4)
			ax2.text(x_pos-delta/1.5, lumo-0.3, layer_material.upper())

			if homo!=0.0:
				homo_shape = [homo,homo,homo_delta,homo_delta]
				ax2.fill(x,homo_shape, color[layer],alpha=0.4)

			layer=layer+1

			n=n+1

		ax1.set_ylabel('Photon density')
		ax1.set_xlabel('Position (nm)')
		ax2.set_ylabel('Energy (eV)')
		ax1.autoscale(enable=True, axis='x', tight=None)
		try:
			t,s = loadtxt(self.optical_mode_file, unpack=True)
			t=t*1e9


			ax1.plot(t,s, 'black', linewidth=3 ,alpha=0.5)
		except:
			print "No mode file\n"

		self.fig.tight_layout()

	def on_changed(self, widget, model):
		cb_text=widget.get_active_text()
		print "Roderick=",cb_text
		if cb_text=="all":
			self.optical_mode_file=self.dump_dir+"/light_1d_photons_tot_norm.dat"
		else:
			self.optical_mode_file=self.dump_dir+"/light_1d_"+cb_text[:-3]+"_photons_norm.dat"
		print self.optical_mode_file
		self.draw_graph(model)
		self.fig.canvas.draw()

	def callback_help(self, widget, data=None):
		cmd = 'firefox http://www.roderickmackenzie.eu/opvdm_wiki.html'
		os.system(cmd)

	def wow(self,exe_command):
		self.main_vbox=gtk.VBox()
		self.gen_main_menu(self,self.main_vbox)
		toolbar = gtk.Toolbar()

		toolbar.set_style(gtk.TOOLBAR_ICONS)
		toolbar.set_size_request(-1, 50)
		self.main_vbox.pack_start(toolbar, False, False, 0)


		self.optical_mode_file=self.dump_dir+"/light_1d_photons_tot_norm.dat"
		self.exe_command=exe_command
		self.edit_list=[]
		self.line_number=[]

		print "loading ",self.config_file
		f = open(self.config_file)
		self.lines = f.readlines()
		f.close()
		pos=0
		for i in range(0, len(self.lines)):
			self.lines[i]=self.lines[i].rstrip()

		n=0
		gui_pos=0
		pos=0
		pos=pos+1	#first comment
		items=int(self.lines[pos])

		layer=0

		for i in range(0, items):
			pos=pos+1
			label=self.lines[pos]	#read label
			print label

			pos=pos+1
			layer_ticknes=self.lines[pos] 	#read thicknes

			pos=pos+1
			layer_material=self.lines[pos] 	#read thicknes

			pos=pos+1
			start=self.lines[pos] 	#read thicknes

			pos=pos+1
			enable_gen=self.lines[pos] 	#read thicknes

			articles.append([ label, str(layer_ticknes),str(layer_material),str(start),enable_gen, True ])

			layer=layer+1

			#print "out -> %s %i",out_text,len(self.edit_list)

			n=n+1

		model = self.__create_model()

		self.cb = gtk.combo_box_new_text()
		self.cb.set_wrap_width(5)
		self.cb_id=self.cb.connect("changed", self.on_changed,model)
		self.update_cb()


		self.fig = Figure(figsize=(5,4), dpi=100)
		canvas = FigureCanvas(self.fig)  # a gtk.DrawingArea

		cid = self.fig.canvas.mpl_connect('button_press_event', self.onclick)

		canvas.figure.patch.set_facecolor('white')
		canvas.set_size_request(900, 400)
		canvas.show()
		self.main_vbox.pack_start(canvas, True, True, 0)
		#attach(canvas, 0, 3, gui_pos, gui_pos+1)

		# create model

		gui_pos=gui_pos+1


		add_button = gtk.Button("Add layer",gtk.STOCK_ADD)
		add_button.show()

		delete_button = gtk.Button("Delete layer",gtk.STOCK_DELETE)
		delete_button.show()

		# create tree view
		treeview = gtk.TreeView(model)
		treeview.set_size_request(500, 150)
		treeview.set_rules_hint(True)
		treeview.get_selection().set_mode(gtk.SELECTION_SINGLE)

		tool_bar_pos=0
		save = gtk.ToolButton(gtk.STOCK_SAVE)
		save.connect("clicked", self.callback_save_image)
		toolbar.insert(save, tool_bar_pos)
		toolbar.show_all()
		tool_bar_pos=tool_bar_pos+1

		image = gtk.Image()
   		image.set_from_file(find_data_file("gui/arrow-right.png"))
		refresh = gtk.ToolButton(image)
		refresh.connect("clicked", self.callback_refresh,treeview)
		toolbar.insert(refresh, tool_bar_pos)
		toolbar.show_all()
		tool_bar_pos=tool_bar_pos+1

		ti_light = gtk.ToolItem()
		lable=gtk.Label("Optical mode:")
		lable.show()
	        ti_hbox = gtk.HBox(False, 2)
		ti_hbox.show()

	        ti_hbox.pack_start(lable, False, False, 0)
	        ti_hbox.pack_start(self.cb, False, False, 0)
		self.cb.show()

		ti_light.add(ti_hbox);
		toolbar.insert(ti_light, tool_bar_pos)
		ti_light.show()

		tool_bar_pos=tool_bar_pos+1


		sep = gtk.SeparatorToolItem()
		sep.set_draw(False)
		sep.set_expand(True)
		toolbar.insert(sep, tool_bar_pos)
		sep.show()
		tool_bar_pos=tool_bar_pos+1


		help = gtk.ToolButton(gtk.STOCK_HELP)
		toolbar.insert(help, tool_bar_pos)
		help.connect("clicked", self.callback_help)
		help.show()
		tool_bar_pos=tool_bar_pos+1

		close = gtk.ToolButton(gtk.STOCK_QUIT)
		close.connect("clicked", self.callback_close,treeview)
		toolbar.insert(close, tool_bar_pos)
		close.show()
		tool_bar_pos=tool_bar_pos+1

	        hbox = gtk.HBox(False, 5)


	        hbox.pack_start(add_button, False, False, 0)
	        hbox.pack_start(delete_button, False, False, 0)
		hbox.show()
		#self.attach(hbox, 2, 4, gui_pos, gui_pos+1,gtk.SHRINK ,gtk.SHRINK)
		self.main_vbox.pack_start(hbox, False, False, 0)
		gui_pos=gui_pos+1



		add_button.connect("clicked", self.on_add_item_clicked, treeview)
		delete_button.connect("clicked", self.on_remove_item_clicked, treeview)


		self.__add_columns(treeview)

		#sw.add()

		#self.attach(treeview, 0, 3, gui_pos, gui_pos+1,gtk.SHRINK ,gtk.SHRINK)
		self.main_vbox.pack_start(treeview, False, False, 0)
		gui_pos=gui_pos+1

		self.connect("delete-event", self.callback_close)

		treeview.show()
		self.add(self.main_vbox)
		self.main_vbox.show()
		self.draw_graph(model)
		self.set_icon_from_file(find_data_file("gui/image.jpg"))
		self.set_title("Optical Model - (www.opvdm.com)")
		self.set_position(gtk.WIN_POS_CENTER)



