# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import pygtk
pygtk.require('2.0')
import gtk
import sys
import os
import shutil
from util import replace
from util import get_token_value
from search import return_file_list
from plot import plot_data
from plot import plot_gen
from plot import plot_info
from util import find_data_file
from about import about_dialog_show
from used_files_menu import used_files_menu
import threading
import gobject
import pyinotify
import multiprocessing
import time
import glob

class _IdleObject(gobject.GObject):

	def __init__(self):
		gobject.GObject.__init__(self)

	def emit(self, *args):
		gobject.idle_add(gobject.GObject.emit,self,*args)

class _FooThread(threading.Thread, _IdleObject):
	__gsignals__ = {
		"file_changed": (
		gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
		}

	def __init__(self, *args):
		threading.Thread.__init__(self)
		_IdleObject.__init__(self)

	def onChange(self,ev):
		file_name=os.path.basename(ev.pathname)
		file_name=file_name.rstrip()
		self.thread_data[0]
		self.thread_data[0]=file_name
		self.emit("file_changed")

	def set_watch_path(self,path,thread_data):
		self.watch_path=path
		self.thread_data=thread_data

	def run(self):
		wm = pyinotify.WatchManager()
		wm.add_watch(self.watch_path, pyinotify.IN_CLOSE_WRITE, self.onChange,False,False)
		self.notifier = pyinotify.Notifier(wm)
		self.notifier.loop()


	def stop(self):
		print "thread2:I have shutdown the thread!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!",threading.currentThread()
		self.notifier.stop()
		print "thread:I have shutdown the thread!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!",threading.currentThread()

class server:
	def __init__(self):
		self.running=False
		self.thread_data=[""]

	def init(self):
		self.cpus=multiprocessing.cpu_count()
		self.jobs=[]
		self.status=[]
		self.jobs_running=0
		self.jobs_run=0

	def setup_gui(self,spin,progress):
		self.spin=spin
		self.progress=progress

	def add_job(self,command):
		self.jobs.append(command)
		self.status.append(0)

	def start(self,sim_dir,exe_command):
		self.progress.show()
		self.spin.start()
		self.sim_dir=sim_dir
		self.exe_command=exe_command
		self.thread = _FooThread()
		self.thread.set_watch_path(self.sim_dir,self.thread_data)
		self.thread.connect("file_changed", self.callback)
		self.thread.daemon = True
		if self.running==True:
			print "thread:I'm still running!!!!!!!!!!!!!"
			self.stop()

		self.running=True
		self.thread.start()
		self.run_jobs()


	def print_jobs(self):
		for i in range(0, len(self.jobs)):
			print self.jobs[i],self.status[i]

	def run_jobs(self):
		if (len(self.jobs)==0):
			return

		for i in range(0, len(self.jobs)):
			if (self.jobs_running<self.cpus):
				if self.status[i]==0:
					self.status[i]=1
					self.jobs_running=self.jobs_running+1
					print "Running job",self.jobs[i]
					print "thread: forked"
					if os.fork() == 0:
						os.chdir(self.jobs[i])
						cmd=self.exe_command
						os.system(cmd)
						print "Finished",self.jobs[i]
						f = open(self.sim_dir+"lock"+str(i)+".dat",'w')
						f.write("hello")
						f.close()

						sys.exit()

	def stop(self):
		self.thread.stop()
		self.jobs=[]
		self.status=[]
		self.jobs_running=0
		self.jobs_run=0
		self.spin.stop()
		self.progress.hide()
		self.progress.set_fraction(0.0)
		self.running=False
		ls=os.listdir(self.sim_dir)

		for i in range(0, len(ls)):
			if ls[i][:4]=="lock" and ls[i][-4:]==".dat":
				del_file=self.sim_dir+'/'+ls[i]
				print "delete file:",del_file
				os.remove(del_file)

		print "thread: I have shut down the server."

	def callback(self,object):
		file_name=self.thread_data[0]
		print file_name
		if str(file_name)>4:
			test=file_name[:4]
			if test=="lock":
				rest=file_name[4:]
				finished=int(os.path.splitext(rest)[0])
				#print self.jobs[finished]
				#self.print_jobs()
				self.jobs_run=self.jobs_run+1
				self.jobs_running=self.jobs_running-1
				self.progress.set_fraction(float(self.jobs_run)/float(len(self.jobs)))
				self.run_jobs()
				if (self.jobs_run==len(self.jobs)):
					self.stop()

