// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <util.h>
#include "true_false.h"
#include "../../device.h"
#include "../../dump_ctrl.h"
#include "../../light.h"

static double last_Psun=-1000.0;
static double last_laser_eff=-1000.0;
static double last_wavelength_laser=-1000.0;

void light_init()
{
last_Psun=-1000.0;
last_laser_eff=-1000.0;
last_wavelength_laser=-1000.0;
}

void light_transfer_gen_rate_to_device(struct device *cell,struct light *in)
{
int i=0;
double Gn=0.0;
double Gp=0.0;

	if (in->align_mesh==FALSE)
	{
		for (i=0;i<cell->ymeshpoints;i++)
		{

			Gn=inter_get_raw(in->x,in->Gn,in->points,in->device_start+cell->ymesh[i])*in->Dphotoneff;
			Gp=inter_get_raw(in->x,in->Gp,in->points,in->device_start+cell->ymesh[i])*in->Dphotoneff;

			cell->Gn[i]=Gn*in->electron_eff;
			cell->Gp[i]=Gp*in->hole_eff;
			cell->Habs[i]=0.0;

		}
	}else
	{
		for (i=0;i<cell->ymeshpoints;i++)
		{
			cell->Gn[i]=in->Gn[in->device_start_i+i]*in->Dphotoneff;
			cell->Gp[i]=in->Gp[in->device_start_i+i]*in->Dphotoneff;//changed to Gp
		}

	}

}

void light_solve_and_update(struct device *cell,struct light *in,double Psun_in,double laser_eff_in)
{
in->Psun=Psun_in;
in->laser_eff=laser_eff_in;

if ((last_laser_eff!=in->laser_eff)||(last_Psun!=in->Psun)||(last_wavelength_laser!=in->laser_wavelength))
{

	light_solve_optical_problem(in);
	last_laser_eff=in->laser_eff;
	last_Psun=in->Psun;
	last_wavelength_laser=in->laser_wavelength;
}

light_dump_1d(in, in->laser_pos,"");

light_transfer_gen_rate_to_device(cell,in);
}
