// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <time.h>
#include <dirent.h>
#include <sys/types.h>
#include <unistd.h>
#include <limits.h>
#include <stdio.h>
#include <sys/stat.h>
#include "util.h"
#include <pwd.h>
#include "sim.h"
#include "dos.h"
#include "server.h"
#include "elec_plugins.h"
#include "light_interface.h"
#include "dump.h"
#include "complex_solver.h"
#include "elec_plugins.h"
#include "license.h"

struct device cell;

void device_init(struct device *in)
{
	in->Voc=0.0;
	in->Jsc=0.0;
	in->FF=0.0;
	in->Pmax=0.0;
	in->Pmax_voltage=0.0;
	device_get_memory(in);

}


int main (int argc, char *argv[])
{
cell.onlypos=FALSE;
#ifndef nolock
antje0();
#endif

//char path[PATH_MAX];
//char dest[PATH_MAX];
//pid_t pid = getpid();
//sprintf(path, "/proc/%d/exe", pid);

//if (readlink(path, dest, PATH_MAX) == -1)
//{
//	printf("error\n");
//	exit(1);
//}
//  char *base = strrchr(dest, '/');
//*base='/';
//*(base+1)=0;
set_plot_script_dir("./");

//set_plot_script_dir(char * in)
if (scanarg( argv,argc,"--help")==TRUE)
{
	printf("opvdm_core - Organic Photovoltaic Device Model\n");
	printf(copyright);
	printf("\n");
	printf("Usage: opvdm_core [options]\n");
	printf("\n");
	printf("Options:\n");
	printf("\n");
	printf("\t--outputpath\tdirectory to run (default ./)\n");
	printf("\t--version\tdisplays the current version\n");
	printf("\n");
	printf("Additional information about opvdm is available at www.opvdm.com.\n");
	printf("\n");
	printf("Report bugs to: roderick.mackenzie@nottingham.ac.uk\n\n");
	exit(0);
}
if (scanarg( argv,argc,"--version")==TRUE)
{
	printf("opvdm_core, Version %s\n",version);
	printf(copyright);
	printf(this_is_free_software);
	printf("There is ABSOLUTELY NO WARRANTY; not even for MERCHANTABILITY or\n");
	printf("FITNESS FOR A PARTICULAR PURPOSE.\n");
	printf("\n");
	exit(0);
}

#ifdef private
if (scanarg( argv,argc,"--patch_file")==TRUE)
{
	char *patch_file=get_arg_plusone( argv,argc,"--patch_file");
	char *patch_path=get_arg_plusone( argv,argc,"--patch_path");
	patch(patch_path,patch_file);
	exit(0);
}


if ((scanarg( argv,argc,"--fit")==TRUE)||(scanarg( argv,argc,"--1fit")==TRUE))
{
set_dump_status(dump_lock, FALSE);
set_dump_status(dump_print_text, FALSE);
set_dump_status(dump_iodump, FALSE);
set_dump_status(dump_optics, FALSE);
set_dump_status(dump_newton, FALSE);
set_dump_status(dump_plot, FALSE);
set_dump_status(dump_stop_plot, FALSE);
set_dump_status(dump_opt_for_fit, FALSE);
set_dump_status(dump_print_newtonerror,FALSE);
set_dump_status(dump_print_converge,FALSE);
set_dump_status(dump_print_pos_error, FALSE);
set_dump_status(dump_lock, TRUE);
}
#endif

if(geteuid()==0) {
	ewe("Don't run me as root!\n");
}
lock_command_line(argc, argv);

set_dump_status(dump_stop_plot, FALSE);
set_dump_status(dump_print_text, TRUE);

set_io_dump(FALSE);
srand(time(0));
textcolor(fg_green);
randomprint("Organic Photovoltaic Device Model (www.opvdm.com)\n");
randomprint(copyright);
#ifdef gnu_copyright
randomprint("You should have received a copy of the GNU General Public License\n");
randomprint("along with this software.  If not, see www.gnu.org/licenses/.\n");
#endif
randomprint("\n");
randomprint("If you wish to collaborate in anyway please get in touch:\n");
randomprint("roderick.mackenzie@nottingham.ac.uk\n");
randomprint("www.roderickmackenzie.eu/contact.html\n");
randomprint("\n");
textcolor(fg_reset);

globalserver.on=FALSE;
globalserver.cpus=1;
globalserver.readconfig=TRUE;

if (scanarg( argv,argc,"--outputpath")==TRUE)
{
	strcpy(sim_output_path(),get_arg_plusone( argv,argc,"--outputpath"));
}else
{
	strcpy(sim_output_path(),"./");
}

#ifdef private
if (scanarg( argv,argc,"--cpus")==TRUE)
{
sscanf(get_arg_plusone( argv,argc,"--cpus"),"%d",&globalserver.cpus);
globalserver.readconfig=FALSE;
}

if (scanarg( argv,argc,"--optics_test")==TRUE)
{
struct light two;
struct light test;
light_init(&two,&cell);
light_init(&test,&cell);
//light_set_dx(&cell.mylight,cell.ymesh[1]-cell.ymesh[0]);
light_load_config(&two,"./");
light_load_config(&test,"./");

set_dump_status(dump_lock, FALSE);
set_dump_status(dump_optics, TRUE);
set_dump_status(dump_optics_verbose, FALSE);

int pos=two.lpoints/2;

two.sun_E[pos]=1.0;
test.sun_E[pos]=1.0;

printf("Solving for %f\n",two.l[pos]*1e9);
light_solve_lam_slice(&two,pos);
//light_test_air_metal(&test,pos);
light_test_air_abs_interface(&test,pos);
light_cal_photon_density(&two);
light_cal_photon_density(&test);
light_dump_1d(&two, pos,"");
light_dump_1d(&test, pos,"_test");
printf("Written data\n");

light_free(&two);
light_free(&test);

return 0;
}

if (scanarg( argv,argc,"--optics")==TRUE)
{
struct light two;
light_init(&two,&cell);
//light_set_dx(&cell.mylight,cell.ymesh[1]-cell.ymesh[0]);
light_load_config(&two,"./");
set_dump_status(dump_lock, FALSE);
set_dump_status(dump_optics, TRUE);
set_dump_status(dump_optics_verbose, FALSE);
light_solve_and_update(&cell,&two,1000.0,0.0);
light_dump(&two);
light_free(&two);
return 0;
}
#endif

if (scanarg( argv,argc,"--onlypos")==TRUE)
{
cell.onlypos=TRUE;
}


char name[1000];
FILE *config=fopena(sim_output_path(),"ver.inp","r");
if (config==NULL)
{
ewe("No ver.inp file, I suspect there are no input files at all\n");
}

fscanf(config,"%s",name);
fclose(config);

if (strcmp(name,version)!=0)
{
printf("Software is version %s and the input files are version %s\n",version,name);
exit(0);
}



server_init(&globalserver);
int ret=0;

#include "main_args.c"
{
	gen_dos_fd_gaus_fd();

	server_add_job(&globalserver,"./");
	print_jobs(&globalserver);
	ret=server_run_jobs(&globalserver);

}

server_shut_down(&globalserver);
if (ret!=0)
{
	return 1;
}
return 0;
}

