// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../../sim.h"
#include "jv.h"
#include "frame.h"
#include "dump.h"
#include <math.h>
#include "../../elec_plugins.h"
#include "ntricks.h"
#include "gui_hooks.h"

static int unused __attribute__((unused));

void sim_jv(struct device *in)
{
gui_start();
struct jv config;
jv_load_config(&config,in);
double V=0.0;
double Vstop=config.Vstop;
double Vstep=config.Vstep;
FILE *iv;
int ittr=0;
double J;
double Pden;
int first=TRUE;
double Vlast;
double Jlast;
double Pdenlast;
double Vexternal;

struct istruct ivexternal;
inter_init(&ivexternal);

struct istruct jvexternal;
inter_init(&jvexternal);

struct istruct charge;
inter_init(&charge);

struct istruct charge_tot;
inter_init(&charge_tot);

struct istruct klist;
inter_init(&klist);

//light_solve_and_update(in,&(in->mylight),in->Psun*config.jv_light_efficiency,0.0);

in->Vapplied=0.0;
if (fabs(config.Vstart-in->Vapplied)>0.2)
{
	ramp_externalv(in,0.0,config.Vstart);
}
in->Vapplied=config.Vstart;
sim_externalv(in,in->Vapplied);

remesh_reset(in,in->Vapplied);

V=in->Vapplied;
newton_set_min_ittr(30);
in->Vapplied=config.Vstart;
newton_sim_jv(in);
newton_set_min_ittr(0);
struct map framen;
struct map framep;
struct map frame;
if (in->dump_movie==TRUE)
{
	frame_init(&framen,in->ymeshpoints,1000,in->ymesh[0],in->ymesh[in->ymeshpoints-1],-7.5,-3.5);
	frame_init(&framep,in->ymeshpoints,1000,in->ymesh[0],in->ymesh[in->ymeshpoints-1],-7.5,-3.5);
	frame_init(&frame,in->ymeshpoints,1000,in->ymesh[0],in->ymesh[in->ymeshpoints-1],-7.5,-3.5);
}

double k_voc=0.0;
double n_voc=0.0;
double r_voc=0.0;
double nsc=0.0;
double n_trap_voc=0.0;
double p_trap_voc=0.0;
double n_free_voc=0.0;
double p_free_voc=0.0;
double np_voc_tot=0.0;
	do
	{


		in->Vapplied=V;

		newton_sim_jv(in);

		J=get_equiv_J(in);
		Vexternal=get_equiv_V(in);


		if (in->dump_movie==TRUE)
		{
		frame_reset(&framen);
		frame_reset(&framep);
		frame_reset(&frame);
		dump_build_2d_charge_frame(&framen,&framep,in);
		dump_write_2d_charge_map(&framen,&framep,in);

		dump_build_2d_charge_single_frame(&frame,in);
		dump_write_2d_charge_single_map(&frame,in);
		}
		//getchar();

		if (ittr>0)
		{

			inter_append(&jvexternal,get_equiv_V(in),get_equiv_J(in));
			inter_append(&ivexternal,get_equiv_V(in),get_equiv_I(in));

		}

		ittr++;

		inter_append(&charge,get_equiv_V(in),get_extracted_np(in));
		inter_append(&charge_tot,get_equiv_V(in),get_np_tot(in));

			Pden=fabs(J*Vexternal);

			//printf("Plotted\n");
			plot_now(in,"jv.plot");


			if (first==FALSE)
			{

				//check if we have crossed 0V
				if ((Vlast<=0)&&(Vexternal>=0.0))
				{
					in->Jsc=Jlast+(J-Jlast)*(0-Vlast)/(V-Vlast);
					nsc=get_extracted_np(in);
					printf("nsc=%le\n",nsc);
					printf("Jsc = %e\n",in->Jsc);
				}

				if ((Jlast<=0)&&(J>=0.0))
				{
					in->Voc=Vlast+(Vexternal-Vlast)*(0-Jlast)/(J-Jlast);
					printf("Voc = %e\n",in->Voc);
					k_voc=get_avg_recom(in)/pow(get_extracted_np(in),2.0);
					r_voc=get_avg_recom(in);
					n_voc=get_extracted_np(in);
					np_voc_tot=get_total_np(in);
					n_trap_voc=get_n_trapped_charge(in);
					n_free_voc=get_free_n_charge(in);
					p_trap_voc=get_p_trapped_charge(in);
					p_free_voc=get_free_p_charge(in);



				}

				if ((Pden>Pdenlast)&&(Vexternal>0.0)&&(J<0.0))
				{
					in->Pmax=Pden;
					in->Pmax_voltage=Vexternal;
				}

				if (Vexternal>Vstop) break;

			}

			if (get_dump_status(dump_iodump)==TRUE)
			{
			printf("V=%lf %lf current = %e mA (%e A/m^2) %le\n",V,Vexternal,get_I(in)/1e-3,J,get_J(in));
			}


			Jlast=J;
			Vlast=Vexternal;
			Pdenlast=Pden;
			first=FALSE;

			slice_check(in);

			V+=Vstep;
			Vstep*=config.jv_step_mul;
			//dialog_set_progress ((in->Vstart+V)/(in->Vstop-in->Vstart));
			if ((Vstep>=0)&&(V>Vstop))
			{
				in->stop=TRUE;
				printf("--------------1\n");
			}

			if ((Vstep<0)&&(V<Vstop))
			{
				in->stop=TRUE;
				printf("--------------2\n");
			}

		if (in->stop==TRUE)
		{
			break;
		}
		inter_append(&klist,get_extracted_np(in),get_avg_recom(in)/(pow(get_extracted_np(in),2.0)));

		stop_start(in);


	}while(1);
//printf("exit\n");

in->FF=fabs(in->Pmax/(in->Jsc*in->Voc));

if (get_dump_status(dump_print_text)==TRUE)
{
	printf("Voc= %lf (V)\n",in->Voc);
	printf("Jsc= %lf (A/m^2)\n",in->Jsc);
	printf("Pmax= %lf (W/m^2)\n",in->Pmax);
	printf("Voltage to get Pmax= %lf (V)\n",in->Pmax_voltage);
	printf("FF= %lf\n",in->FF*100.0);
	printf("Efficiency= %lf percent\n",mod(in->Pmax/in->Psun)*100.0);
}

if (get_dump_status(dump_iodump)==TRUE)
{
	FILE *out;
	out=fopena(in->outputpath,"./sim_info.dat","w");
	fprintf(out,"#ff\n%le\n",in->FF);
	fprintf(out,"#pce\n%le\n",mod(in->Pmax/(in->Psun))*100.0);
	fprintf(out,"#voc\n%le\n",in->Voc);
	fclose(out);

	FILE *outfile=fopena(in->outputpath,"./jv_psun_voc.dat","w");
	fprintf(outfile,"%le %le\n",in->Psun,in->Voc);
	fclose(outfile);

	outfile=fopena(in->outputpath,"./jv_voc_nt.dat","w");
	fprintf(outfile,"%le %le\n",in->Voc,n_trap_voc);
	fclose(outfile);

	outfile=fopena(in->outputpath,"./jv_voc_pt.dat","w");
	fprintf(outfile,"%le %le\n",in->Voc,p_trap_voc);
	fclose(outfile);

	outfile=fopena(in->outputpath,"./jv_psun_nf_nt.dat","w");
	fprintf(outfile,"%le %le\n",in->Psun,n_free_voc/n_trap_voc);
	fclose(outfile);

	outfile=fopena(in->outputpath,"./jv_psun_pf_pt.dat","w");
	fprintf(outfile,"%le %le\n",in->Psun,p_free_voc/p_trap_voc);
	fclose(outfile);

	outfile=fopena(in->outputpath,"./jv_psun_np_tot.dat","w");
	fprintf(outfile,"%le %le\n",in->Psun,np_voc_tot);
	fclose(outfile);


	outfile=fopena(in->outputpath,"./jsc.dat","w");
	fprintf(outfile,"%le",in->Jsc);
	fclose(outfile);

	outfile=fopena(in->outputpath,"./nsc.dat","w");
	fprintf(outfile,"%le",nsc);
	fclose(outfile);

	outfile=fopena(in->outputpath,"./vbi.dat","w");
	fprintf(outfile,"%le",in->vbi);
	fclose(outfile);

	outfile=fopena(in->outputpath,"./jv_gen.dat","w");
	fprintf(outfile,"%le",get_avg_gen(in));
	fclose(outfile);

	inter_save_a(&klist,in->outputpath,"k.dat");
	inter_free(&klist);
	}

FILE *kvocfile=fopena(in->outputpath,"./kvoc.dat","w");
fprintf(kvocfile,"%le %le\n",n_voc,k_voc);
fclose(kvocfile);

kvocfile=fopena(in->outputpath,"./voc_tau.dat","w");
fprintf(kvocfile,"%le %le\n",in->Voc,n_voc/r_voc);
fclose(kvocfile);

kvocfile=fopena(in->outputpath,"./n_tau.dat","w");
fprintf(kvocfile,"%le %le\n",n_voc,1.0/(k_voc*n_voc));
fclose(kvocfile);

inter_save_a(&charge,in->outputpath,"charge.dat");
inter_free(&charge);

inter_save_a(&charge_tot,in->outputpath,"charge_tot.dat");
inter_free(&charge_tot);

inter_save_a(&ivexternal,in->outputpath,"ivexternal.dat");
inter_free(&ivexternal);

inter_save_a(&jvexternal,in->outputpath,"jvexternal.dat");
inter_free(&jvexternal);


if (in->dump_movie==TRUE)
{
	frame_free(&framen);
	frame_free(&framep);
	frame_free(&frame);
}

gui_stop();
}




void jv_load_config(struct jv* in,struct device *dev)
{
double ver=0.0;
char name[200];
FILE *config=fopena(dev->outputpath,"jv.inp","r");

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->Vstart));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->Vstop));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->Vstep));

config_read_line_to_double(&(in->jv_step_mul),config,"#jv_step_mul");

config_read_line_to_double(&(in->jv_light_efficiency),config,"#jv_light_efficiency");
in->jv_light_efficiency=fabs(in->jv_light_efficiency);

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(ver));
if (ver!=1.2)
{
ewe("File compatability problem %s\n","jv.inp");
}

unused=fscanf(config,"%s",name);

if (strcmp("#end",name)!=0)
{
ewe("Problem with jv.inp file last item read %s!\n",name);
}
fclose(config);
}
