// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <i.h>
#include <math.h>
#include "../../sim.h"
#include "../../dump.h"
#include "../../slice.h"
#include "../../ntricks.h"
#include "shg.h"

static int unused __attribute__((unused));

struct shg shg_config;

double shg_stark(struct device *in)
{
	struct istruct E;
	struct istruct phi;
	inter_import_array(&E,in->ymesh, in->phi,in->ymeshpoints,TRUE);
	inter_import_array(&phi,in->ymesh, in->phi,in->ymeshpoints,TRUE);
	if (get_dump_status(dump_iodump)==TRUE) inter_save_a(&E,in->outputpath,"shg_phi.dat");
	inter_deriv(&E,&phi);
	if (get_dump_status(dump_iodump)==TRUE) inter_save_a(&E,in->outputpath,"shg_E.dat");
	inter_pow(&E,2.0);
	inter_mul(&E,shg_config.shg_stark);
	if (get_dump_status(dump_iodump)==TRUE) inter_save_a(&E,in->outputpath,"shg_x_stark.dat");
	double sum=inter_avg(&E);//inter_avg_range(&E,50e-9,70e-9);
	inter_free(&E);
	inter_free(&phi);

return sum;
}


double shg_get_light_den(struct device *in)
{
int i=0;
double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	tot+=in->Gn[i];
}
tot/=(double)in->ymeshpoints;

return tot;
}


void sim_shg(struct device *in)
{
shg_load_config(&shg_config,in);
int i;


//getchar();
in->dt=shg_config.shg_big_dt;

carrier_count_reset(in);

in->go_time=FALSE;
double t0=shg_config.shg_t0;							//apply voltage
double t1=shg_config.shg_t0+shg_config.t_big_offset;					//start of experiment
double t2=shg_config.shg_t0+shg_config.t_big_offset+shg_config.shg_tstart;			//laser pulse
double t3=shg_config.shg_t0+shg_config.t_big_offset+shg_config.shg_tstart+shg_config.shg_stop;	//experiment stop
printf("Running shg. %e\n",shg_config.shg_externalv);
int ittr=0;
int pittr=-1;

light_solve_and_update(in,&(in->mylight),in->Psun,0.0);
double V=shg_config.shg_externalv;

if (fabs(V-in->Vapplied)>0.2)
{
	ramp(in,0.0,V,7.0);
}

//newton_sim_shg(in,V,TRUE);


//sim_externalv(in,shg_config.shg_externalv);

//////////////////

sim_externalv(in,V);

for (i=0;i<4;i++)
{
newton_sim_shg(in,V,FALSE);
printf("%le\n",in->Vapplied);
device_timestep(in);
}

//////////////////
//newton_sim_shg(in);

time_init(in);

for (i=0;i<in->ymeshpoints;i++)
{
in->n_orig[i]=in->n[i]+in->nt_all[i];
in->p_orig[i]=in->p[i]+in->pt_all[i];

in->n_orig_f[i]=in->n[i];
in->p_orig_f[i]=in->p[i];

in->n_orig_t[i]=in->nt_all[i];
in->p_orig_t[i]=in->pt_all[i];
}




device_timestep(in);

in->go_time=TRUE;

in->time=0.0;

	FILE *out=fopen("./shg.dat","w");
	fclose(out);


struct istruct shgoutj;
inter_init(&shgoutj);

struct istruct shgoutstark;
inter_init(&shgoutstark);

struct istruct field;
inter_init(&field);

struct istruct shg_vapplied;
inter_init(&shg_vapplied);

dump_dynamic_init(in);
double stark_orig=shg_stark(in);
printf("Main shg loop\n");
double stark=0.0;
reset_npinit(in);
int big_step=FALSE;
double current=0.0;
do
{
//newton_sim_stark(in);
current=newton_sim_shg(in,V,TRUE);
device_timestep(in);

	if (in->time<t0)
	{
		V=shg_config.shg_externalv;
		big_step=TRUE;
	}else
	if (in->time<(t1-shg_config.shg_dt))
	{
		V=shg_config.shg_onvoltage;
		big_step=TRUE;
	}else
	{
		if (big_step==TRUE)
		{
		V=shg_config.shg_onvoltage;
		big_step=FALSE;
		in->dt=shg_config.shg_dt;
		}
	}

	//getchar();
//printf("%le\n",in->Vapplied);
if (big_step==FALSE)
{

	if (in->time>t2) pittr++;

	if (pittr==0)
	{

		if (shg_config.shg_laser_on==TRUE)
		{
			light_solve_and_update(in,&(in->mylight),in->Psun,shg_config.shg_lasereff);
		}
		for (i=0;i<in->ymeshpoints;i++)
		{
			in->Gn[i]*=in->mylight.pulseL/in->dt;
			in->Gp[i]*=in->mylight.pulseL/in->dt;
		}
	}

	if (pittr==1)
	{

			for (i=0;i<in->ymeshpoints;i++)
			{
				in->Gn[i]=0.0;
				in->Gp[i]=0.0;
			}
	}

	dump_dynamic_add_data(in);
	slice_check(in);
	stark=shg_stark(in);

	inter_append(&shgoutstark,in->time-t2,stark);
}

	inter_append(&shg_vapplied,in->time,in->Vapplied);
	inter_append(&field,in->time,get_avg_field(in));


	ittr++;

	plot_now(in,"shg.plot");


	inter_append(&shgoutj,in->time,get_J(in));

	if (get_dump_status(dump_print_text)==TRUE)
	{
		printf("shg step t=%.8le V=%le current=%le VRshg=%le %le\n",in->time,V,current,current*shg_config.shg_R,in->last_error);
	}


	stop_start(in);

	if (in->time>t2)
	{
		in->dt*=shg_config.shg_dt_mull;
	}

//printf("%le %le %le %le %le %ld\n",t0,t1,t2,t3,in->time,big_step);
	if (in->time>t3) break;

	if (in->stop==TRUE)
	{
		break;
	}


}while(1);
dump_dynamic_save(in);
dump_dynamic_free(in);

in->go_time=FALSE;

char outpath[1000];


sprintf(outpath,"%s%s",in->outputpath,"shgj.dat");
if (get_dump_status(dump_iodump)==TRUE) inter_save(&shgoutj,outpath);

sprintf(outpath,"%s%s",in->outputpath,"shg_stark.dat");
inter_save(&shgoutstark,outpath);


sprintf(outpath,"%s%s",in->outputpath,"shg_stark_simple.dat");
inter_div_double(&shgoutstark,stark_orig);
inter_save(&shgoutstark,outpath);

sprintf(outpath,"%s%s",in->outputpath,"shg_field.dat");
inter_save(&field,outpath);

sprintf(outpath,"%s%s",in->outputpath,"shg_vapplied.dat");
inter_save(&shg_vapplied,outpath);


inter_free(&shgoutj);
inter_free(&shgoutstark);
inter_free(&field);
inter_free(&shg_vapplied);
}


void shg_load_config(struct shg *in,struct device *dev)
{
double ver=0.0;
char name[200];
FILE *config=fopena(dev->outputpath,"shg.inp","r");

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_tstart));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_1start));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_1stop));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_2start));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_2stop));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_den));
in->shg_den=fabs(in->shg_den);
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_externalv));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_dt));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_dt_mull));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_stop));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_stark));
in->shg_stark=fabs(in->shg_stark);
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(in->shg_genprofile));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_lasereff));
in->shg_lasereff=fabs(in->shg_lasereff);
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_onvoltage));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d",&(in->shg_laser_on));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_R));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_t0));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_t1));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_t2));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->t_big_offset));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->shg_big_dt));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(ver));
if (ver!=1.2)
{
ewe("File compatability problem %s \n","shg.inp");
}

unused=fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
ewe("Problem with tof.inp file last item read %s!\n",name);
}
fclose(config);
}
