// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../dump.h"
#include "../../slice.h"
#include "../../sim.h"
#include "shg.h"

static double glob_wanted=0.0;
static int glob_use_cap=0;

void newton_aux_shg(struct device *in,double V,double* i,double* didv,double* didphi,double* didxil,double* didxipl,double* didphir,double* didxir,double* didxipr)
{
//<clean>
double Rshort_shg=shg_config.shg_R;
double Rcontact=in->Rcontact;
double C=in->C;
if (glob_use_cap==FALSE) C=0.0;//1e-1;
//C=0.0;
double i0=*i;
double didv0=*didv;
double didphi0=*didphi;
double didxil0=*didxil;
double didxipl0=*didxipl;
double didphir0=*didphir;
double didxir0=*didxir;
double didxipr0=*didxipr;

*i=glob_wanted-(V+((Rcontact+Rshort_shg)*(i0+V/in->Rshunt+C*(V-in->Vapplied_last)/in->dt)));
*didv=-(1.0+((Rcontact+Rshort_shg)*(didv0+1.0/in->Rshunt+C*(1.0)/in->dt)));
*didphi=-((Rcontact+Rshort_shg)*didphi0);
*didxil=-((Rcontact+Rshort_shg)*didxil0);
*didxipl=-((Rcontact+Rshort_shg)*didxipl0);
*didphir=-((Rcontact+Rshort_shg)*didphir0);
*didxir=-((Rcontact+Rshort_shg)*didxir0);
*didxipr=-((Rcontact+Rshort_shg)*didxipr0);
//</clean>
return;
}

double newton_sim_shg(struct device *in,double Vtot,int usecap)
{
//<clean>
double C=in->C;
in->newton_cur=TRUE;
solver_realloc(in);
glob_wanted=Vtot;
glob_use_cap=usecap;
in->newton_aux=&newton_aux_shg;
solve_all(in);
if (glob_use_cap==FALSE) C=0.0;
return get_I(in)+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt;
//</clean>
}
